// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Dynamic object loader.
//
// Do all OS-dependent tuning here!
// For now it is tested only in FreeBSD 2.2.1.
//
// FreeBSD notes:
//  It is known, that it's impossible to statically link executables,
//  when this code is in use, because in FreBSD dlopen/dlclose is not
//  library's functions, but internal dynamic loader interface. And so
//  when there is no dynamic loader compiled in there is no required
//  symbols present.
//
#include <stdio.h>
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// Constructor (and loading on demand)
//
easysql_dynamic_loader::easysql_dynamic_loader(char const *file)
                       :easysql_error_reporting("dynaloader",D_MISC)
{ load(file);
}

///////////////////////////////////////////////////////////////////////////////
// Destructor
//
easysql_dynamic_loader::~easysql_dynamic_loader()
{ unload();
}

///////////////////////////////////////////////////////////////////////////////
// The rest is OS dependent and is placed in subdirectory dl/ on
// file-per-os basis.
//
#if defined(__FreeBSD__)
#include "dl/freebsd.C"
#elif defined(__linux__)
#include "dl/linux.C"

///////////////////////////////////////////////////////////////////////////////
// The following is fake dynamic loader for unknown systems and system
// unable to load modules on the fly
//
#else
#warning Dynamic loading not supported for your OS

///////////////////////////////////////////////////////////////////////////////
// Loading (opening) shared object
//
bool
easysql_dynamic_loader::load(char const *file)
{ set_error("Can't load `%s' -- dynamic loading not supported",file);
  return false;
}

///////////////////////////////////////////////////////////////////////////////
// Unloading (closing) shared object
//
bool
easysql_dynamic_loader::unload()
{ return true;
}

///////////////////////////////////////////////////////////////////////////////
// Checking object
//
easysql_dynamic_loader::operator bool()
{ return false;
}

///////////////////////////////////////////////////////////////////////////////
// Returning symbol address from shared object
//
void *
easysql_dynamic_loader::symbol(char const *name)
{ set_error("Can't find `%s' -- dynamic loading not supported",name);
  return NULL;
}
#endif		// OS

///////////////////////////////////////////////////////////////////////////////
R("$Id: dynaload.C,v 1.3 1997/10/23 18:40:33 am Exp $");
