// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Error reporting base class able to process symbols of formatting.
//
#include <stdio.h>
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// Constructing empty object
//
easysql_error_reporting::easysql_error_reporting(char const *c, unsigned lvl)
                        :category(strDup(c))
                        ,text(NULL)
                        ,size(0)
                        ,level(lvl)
{
}

///////////////////////////////////////////////////////////////////////////////
// Freeing string
//
easysql_error_reporting::~easysql_error_reporting()
{ if(text)
   Free(text);
  if(category)
   Free(category);
}

///////////////////////////////////////////////////////////////////////////////
// Registering error message
//
void
easysql_error_reporting::set_error(char const *msg, ...)
{ if(!msg)
   { no_error();
     return;
   }
  if(!text)
   { text=(typeof(text))mAlloc(sizeof(*text)*(size=256));
     *text=0;
   }

  va_list ap;
  va_start(ap,msg);
  while(1)
   { if(unsigned(vsnprintf(text,size,msg,ap))<size)
      break;
     text=(typeof(text))reAlloc(text,sizeof(*text)*(size+=256));
   }
  va_end(ap);
  log(D_ERROR|level,"%s: %s",category ? category : "reporting",text);
}

///////////////////////////////////////////////////////////////////////////////
// Clearing error message
//
void
easysql_error_reporting::no_error()
{ if(text) *text=0;
}

///////////////////////////////////////////////////////////////////////////////
// Returning error text or NULL in case there was no error reported
//
char const *
easysql_error_reporting::error_text()
{ return text && *text ? text : (char const *)NULL;
}
